c    "Copyright 2007 Prof. Robert Dalrymple, Prof. M. Gomez Gesteira, Dr Benedict Rogers, 
c     Alejandro Crespo, Muthukumar Narayanaswamy, Dr Shan Zou, Dr Andrea Panizzo "
c
c    This file is part of SPHYSICS.
c
c    SPHYSICS is free software; you can redistribute it and/or modify
c    it under the terms of the GNU General Public License as published by
c    the Free Software Foundation; either version 3 of the License, or
c    (at your option) any later version.
c
c    SPHYSICS is distributed in the hope that it will be useful,
c    but WITHOUT ANY WARRANTY; without even the implied warranty of
c    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c    GNU General Public License for more details.
c
c    You should have received a copy of the GNU General Public License
c    along with this program.  If not, see <http://www.gnu.org/licenses/>.

      subroutine celij(j1,j2,kind,kind_p1,ini_kind_p2)
c
      include 'common.2D'  
      
      do kind_p2=ini_kind_p2,2
        if(nc(j2,kind_p2).ne.0) then

        is1 = ist(j1,kind_p1)
        ie1 = ien(j1,kind_p1)
        
        do ii=is1,ie1
         i = ip(ii,kind_p1)
         is2 = ist(j2,kind_p2)
         ie2 = ien(j2,kind_p2)

         do jj=is2,ie2
            j = ip(jj,kind_p2)
            drx = xp(i) - xp(j)
            drz = zp(i) - zp(j)
            rr2 = drx*drx + drz*drz

	     if(rr2.le.fourh2(kind).and.rr2.ge.1.e-18) then
             dux = up(i) - up(j)
             duz = wp(i) - wp(j)

c            Calculating kernel & Normalized Kernel
             call kernel(drx,drz,i,j,j1,j2,kind,rr2)   
               
c            ...  average density

             robar  = 0.5*( rhop(i) + rhop(j) )
             one_over_rhobar = 2.0/(rhop(i) + rhop(j))
c
             cbar = 0.5*( cs(i)   + cs(j)   ) 

c            ...  inner product rv
c
             dot = drx*dux + drz*duz

c            Used to calculate the time step duu to viscosity

             visc_dt=max(dot/(rr2 + eta2),visc_dt) 
c
c            ...  pressure and viscous force (Monaghan 1992; Ann. Rev.
c			        Astron. Astrop. 30. Formula 3.3)
c                 pm(j) is mass of particle j
c
             p_v = pr(i) + pr(j)


             if (index_tensile.eq.1) then

c   ____ Tensile correction 
 
               fab=Wab*od_Wdeltap(kind) !NOTE: We'll use a non-normalized
               fab=fab*fab				!kernel to calculate tensile correction
               fab=fab*fab   !  It's the (Wab/Wdeltap)**4  of Monaghan's paper

               if (p(i).gt.0) then
                  Ra= 0.01 *pr(i)
               else
                  Ra= 0.2 *abs(pr(i))
               endif

               if (p(j).gt.0) then
                  Rb= 0.01 *pr(j)
               else
                  Rb= 0.2 *abs(pr(j))
               endif

               R=Ra+Rb
               p_v = p_v+ R*fab

             endif


	      if(i.gt.nb.and.j.gt.nb) then   !both fluid particles

               ax(i) = ax(i) - pm(j) * p_v * frx
               az(i) = az(i) - pm(j) * p_v * frz

               ax(j) = ax(j) + pm(i) * p_v * frx
               az(j) = az(j) + pm(i) * p_v * frz


c              --- CSPH terms ---

               Vj = pVol(j)   !=pm(j)/rhop(j)

               frx_norm_i = Vj*frx   !/sum_dWdx(i)
               frz_norm_i = Vj*frz   !/sum_dWdz(i)

               dudx_CSPH(i) = dudx_CSPH(i) - dux*frx_norm_i
               dudz_CSPH(i) = dudz_CSPH(i) - dux*frz_norm_i
               dwdx_CSPH(i) = dwdx_CSPH(i) - duz*frx_norm_i
               dwdz_CSPH(i) = dwdz_CSPH(i) - duz*frz_norm_i
 
               Vi = pVol(i)   !=pm(i)/rhop(i)

               frx_norm_j = Vi*frx   !/sum_dWdx(j)
               frz_norm_j = Vi*frz   !/sum_dWdz(j)

               dudx_CSPH(j) = dudx_CSPH(j) - dux*frx_norm_j
               dudz_CSPH(j) = dudz_CSPH(j) - dux*frz_norm_j
               dwdx_CSPH(j) = dwdx_CSPH(j) - duz*frx_norm_j
               dwdz_CSPH(j) = dwdz_CSPH(j) - duz*frz_norm_j

               call viscosity(dot,drx,drz,dux,duz,kind,rr2,
     +                  cbar,robar,one_over_rhobar,i,j,j1,j2,term2)


c   ...         Thermal Energy

               term1=0.5 * p_v *( frx*dux+frz*duz)

               aTE(i)=aTE(i)+pm(j) * (term1+term2)
               aTE(j)=aTE(j)+pm(i) * (term1+term2)


c              ...  density acceleration
c              using the derivative of the kernel, not the kernel itself

               dot2 = dux*frx + duz*frz
               ar(i) = ar(i) + pm(j)*dot2
               ar(j) = ar(j) + pm(i)*dot2

c
c              ...  XSPH correction (Monaghan 1994;  J. Comp. Phys. 110. Formula 2.6)
c

               pmj_Wab_over_rhobar = pm(j)*Wab*one_over_rhobar
               ux(i) = ux(i) - dux*pmj_Wab_over_rhobar  !pm(j) * dux * Wab / robar ! (2.6)
               wx(i) = wx(i) - duz*pmj_Wab_over_rhobar  !pm(j) * duz * Wab / robar

               pmi_Wab_over_rhobar = pm(j)*Wab*one_over_rhobar
               ux(j) = ux(j) + dux*pmi_Wab_over_rhobar   !pm(i) * dux * Wab / robar
               wx(j) = wx(j) + duz*pmi_Wab_over_rhobar   !pm(i) * duz * Wab / robar

c              ...  Vorticity calculation


               if(ivort_calc.eq.1.and.i.gt.nb.and.j.gt.nb)then
                if(sum_dWdx(i).gt.0.0001.and.sum_dWdz(i).gt.0.0001)then
c                  V_tempj = pm(j)/rhop(i) 
c	            V_tempi = pm(i)/rhop(j)


c                 curl3xi = duy*frz/sum_dWdz(i)
c     &                 - duz*fry/sum_dWdy(i)
c                 curl3yi = duz*frx/sum_dWdx(i)
c     &                 - dux*frz/sum_dWdz(i)
c                 curl3zi = dux*fry/sum_dWdy(i)
c     &                 - duy*frx/sum_dWdx(i)
  
c                 curl3xj = duy*frz/sum_dWdz(j)
c     &                 - duz*fry/sum_dWdy(j)
c                 curl3yj = duz*frx/sum_dWdx(j)
c     &                 - dux*frz/sum_dWdz(j)
c                 curl3zj = dux*fry/sum_dWdy(j)
c     &                 - duy*frx/sum_dWdx(j)
                
c                 vortx(i) = vortx(i) + V_tempj*curl3xi   !Vorticity in an x=const plane
c                 vorty(i) = vorty(i) + V_tempj*curl3yi   !Vorticity in an y=const plane
c                 vortz(i) = vortz(i) + V_tempj*curl3zi   !Vorticity in an z=const plane
c                 vortx(j) = vortx(j) + V_tempi*curl3xj   !Vorticity in an x=const plane
c                 vorty(j) = vorty(j) + V_tempi*curl3yj   !Vorticity in an y=const plane
c                 vortz(j) = vortz(j) + V_tempi*curl3zj   !Vorticity in an z=const plane

                  curl3y_a(i)= curl3y_a(i)+duz*frx*pm(j)/rhop(i)
                  curl3y_b(i)= curl3y_b(i)+dux*frz*pm(j)/rhop(i)

                  curl3y_a(j)= curl3y_a(j)+duz*frx*pm(i)/rhop(j)
                  curl3y_b(j)= curl3y_b(j)+dux*frz*pm(i)/rhop(j)

 	         endif 
               endif  
		   			
             else if(i.gt.nb.and.j.le.nb)then			!j is boundary particle
               call MonaghanBC(i,j,drx,drz,dot,dux,duz,
     &                          fxbMON,fzbMON)   
               ax(i) = ax(i) + fxbMON   !*uno_sum_wab_i
               az(i) = az(i) + fzbMON
               temp = pVol(i)*visc_wall*
     &           ((drx*frx+drz*frz)/(rr2 + eta2))
               ax(i) = ax(i) + temp*dux
               az(i) = az(i) + temp*duz

             else if(j.gt.nb.and.i.le.nb)then			!i is boundary particle
               call MonaghanBC(j,i,-drx,-drz,dot,-dux,-duz,
     &                          fxbMON,fzbMON)   
               ax(j) = ax(j) + fxbMON   !*uno_sum_wab_i
               az(j) = az(j) + fzbMON
               temp = pVol(j)*visc_wall*
     &         ((drx*frx+drz*frz)/(rr2 + eta2))
               ax(j) = ax(j) - temp*dux
               az(j) = az(j) - temp*duz

             endif  ! Interaction fluid-fluid or fluid-boundary
		  
           endif ! if q<2
         enddo ! Box jj
        enddo ! Box ii
       endif  ! Box jj is not empty
      enddo   ! Kind of particle

      end